package ice.zxing;

//next line added by johan schoeman
import ice.zxing.camera.CameraManager;
import ice.zxing.decoding.CaptureActivityHandler;
import ice.zxing.decoding.InactivityTimer;
import ice.zxing.view.ViewfinderView;
import ice.zxing.camera.CameraManager;
import ice.zxing.decoding.Intents;
import ice.zxing.b4aZXingLib;

import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.util.Vector;
//next line added by johan  schoeman
import android.view.KeyEvent; 
import android.app.Activity;
import android.content.Intent;
import android.graphics.Bitmap;
import android.os.Bundle;
import android.os.Handler;
import android.util.AttributeSet;
import android.util.Log;
import android.view.Gravity;
import android.view.SurfaceHolder;
import android.view.MotionEvent;
import android.view.SurfaceHolder.Callback;
import android.view.SurfaceView;
import android.widget.FrameLayout;
import android.widget.FrameLayout.LayoutParams;
import android.media.ToneGenerator;
import android.media.AudioManager;
import android.os.Vibrator;
import anywheresoftware.b4a.BA.Hide;
import anywheresoftware.b4a.BA;


import com.google.zxing.BarcodeFormat;
import com.google.zxing.Result;
@Hide
public class CaptureActivity extends Activity implements Callback {

	private CaptureActivityHandler handler;
	private ViewfinderView viewfinderView;
	private SurfaceView preview_view;
	private boolean hasSurface;
	private Vector<BarcodeFormat> decodeFormats;
	private String characterSet;
	private InactivityTimer inactivityTimer;
        //next line added by johan schoeman
        private CameraManager mycam;
        private boolean flashIsOn = false;
        private ToneGenerator tg;
        private Vibrator v;

	@Override
	public void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		FrameLayout fl=new FrameLayout(this);
		preview_view=new SurfaceView(this);
		FrameLayout.LayoutParams lps=new LayoutParams(-1, -1);
		lps.gravity=Gravity.CENTER;
		fl.addView(preview_view,lps);
		AttributeSet attrs = null;
		viewfinderView=new ViewfinderView(this, attrs);
		lps=new LayoutParams(-1,-1);
		fl.addView(viewfinderView, lps);
		setContentView(fl);
		CameraManager.init(getApplication());
		hasSurface = false;
		inactivityTimer = new InactivityTimer(this);
	}

	@Override
	protected void onResume() {
		super.onResume();
		SurfaceView surfaceView = preview_view;
		SurfaceHolder surfaceHolder = surfaceView.getHolder();
		if (hasSurface) {
			initCamera(surfaceHolder);
		} else {
			surfaceHolder.addCallback(this);
			surfaceHolder.setType(SurfaceHolder.SURFACE_TYPE_PUSH_BUFFERS);
		}
		decodeFormats = null;
		characterSet = "ISO-8859-1";

	}

	@Override
	protected void onPause() {
		super.onPause();
		if (handler != null) {
			handler.quitSynchronously();
			handler = null;
		}
		CameraManager.get().closeDriver();
	}

	@Override
	protected void onDestroy() {
		inactivityTimer.shutdown();
		super.onDestroy();
	}

// start of added by johan schoeman
   @Override
   @Hide
  public boolean onTouchEvent(MotionEvent event) {
    float eventX = event.getX();
    float eventY = event.getY();

    if (b4aZXingLib.useFrontCam == false) {
      switch (event.getAction()) {
      case MotionEvent.ACTION_DOWN:
        if (flashIsOn == false) {
            flashIsOn = true;
            mycam.myTorchParametersOn();
        } else {
            flashIsOn = false;
            mycam.myTorchParametersOff();
        }
        return true;
      case MotionEvent.ACTION_UP:
        break;
      default:
        return false;
      }  //end of switch
    }
      return true;
  }

   @Override
   public boolean onKeyDown(int keyCode, KeyEvent event) {
     if (b4aZXingLib.useFrontCam == false) {
       switch (keyCode) {
         case KeyEvent.KEYCODE_VOLUME_DOWN:
           if (flashIsOn == true) {
               flashIsOn = false;
               mycam.myTorchParametersOff();
           }                
           return true;
         case KeyEvent.KEYCODE_VOLUME_UP:
           if (flashIsOn == false) {
               flashIsOn = true;
               mycam.myTorchParametersOn();
           }
           return true;
       }
     }
     return super.onKeyDown(keyCode, event);
   }

// end of added by johan schoeman

	private void initCamera(SurfaceHolder surfaceHolder) {
		try {
			CameraManager.get().openDriver(surfaceHolder);
		} catch (IOException ioe) {
			return;
		} catch (RuntimeException e) {
			return;
		}
		if (handler == null) {
			handler = new CaptureActivityHandler(this, decodeFormats,
					characterSet);
		}
	}

	@Override
	public void surfaceChanged(SurfaceHolder holder, int format, int width,
			int height) {

	}

	@Override
	public void surfaceCreated(SurfaceHolder holder) {
		if (!hasSurface) {
			hasSurface = true;
			initCamera(holder);
		}

	}

	@Override
	public void surfaceDestroyed(SurfaceHolder holder) {
		hasSurface = false;

	}

	public ViewfinderView getViewfinderView() {
		return viewfinderView;
	}

	public Handler getHandler() {
		return handler;
	}

	public void drawViewfinder() {       
		viewfinderView.drawViewfinder();
	}
	
	public static class IsChineseOrNot {  
	    
	    public static final boolean isChineseCharacter(String chineseStr) {    
	        char[] charArray = chineseStr.toCharArray();    
	        for (int i = 0; i < charArray.length; i++) {       
	            if ((charArray[i] >= '\u0000' && charArray[i] < '\uFFFD')||((charArray[i] > '\uFFFD' && charArray[i] < '\uFFFF'))) {    
	                continue;  
	            }  
	            else{  
	                return false;  
	            }  
	        }    
	        return true;    
	    } 	    
	} 

	@Override
	protected void onActivityResult(int requestCode, int resultCode, Intent data) {
		// TODO Auto-generated method stub
		super.onActivityResult(requestCode, resultCode, data);
	}

	public void handleDecode(Result obj, Bitmap barcode) {
		inactivityTimer.onActivity();
		viewfinderView.drawResultBitmap(barcode);
		Log.i("B4A", obj.getBarcodeFormat().toString()+":"+obj.getText()+"###"+b4aZXingLib.en+"_result");
		Intent data=new Intent();
		data.putExtra("atype", obj.getBarcodeFormat().toString());
//		data.putExtra("value", obj.getText());
		String UTF_Str="";  
                String GB_Str="";  
                String resultStr=obj.getText();
                boolean is_cN=false; 
        try {  
             UTF_Str=new String(resultStr.getBytes("ISO-8859-1"),"UTF-8");   
            is_cN=IsChineseOrNot.isChineseCharacter(UTF_Str);  
            
            if(!is_cN){  
                GB_Str=new String(resultStr.getBytes("ISO-8859-1"),"GB2312");  
                //data.putExtra("value", GB_Str);
                data.putExtra("value", obj.getText());
            }
            else{
            	data.putExtra("value", resultStr);
            }
        } catch (UnsupportedEncodingException e) {  
            // TODO Auto-generated catch block  
        	
            e.printStackTrace(); 
        }  
                   if (b4aZXingLib.mustVibrate == true) {                              //added by johan schoeman - option to vibrate
                     Vibrator v = (Vibrator) getSystemService(VIBRATOR_SERVICE);
                     if (v.hasVibrator()) {
                       v.vibrate(400);
                     }
                   }  

                   if (b4aZXingLib.mustBeep == true) {                                 //added by johan schoeman - option to beep
                     final ToneGenerator tg = new ToneGenerator(AudioManager.STREAM_NOTIFICATION, 300);
                     tg.startTone(ToneGenerator.TONE_CDMA_ABBR_ALERT);  
		     setResult(RESULT_OK, data);
                   }
                 

		//zx.Ret(obj.getBarcodeFormat().toString(),obj.getText(),barcode);
		finish();
        
         }

}