package ice.zxing.camera;

import android.content.Context;
import android.graphics.Point;
import android.graphics.Rect;
import android.hardware.Camera;
import android.hardware.Camera.Parameters;
import android.os.Build;
import android.os.Handler;
import android.util.Log;
import android.view.SurfaceHolder;
import anywheresoftware.b4a.BA.Hide;
import anywheresoftware.b4a.BA;
import java.io.IOException;
import ice.zxing.b4aZXingLib;

@Hide
public final class CameraManager
{
  private static final String TAG = CameraManager.class.getSimpleName();
  private static final int MIN_FRAME_WIDTH = 240;
  private static final int MIN_FRAME_HEIGHT = 240;

  private static final int MAX_FRAME_WIDTH = 480;
  private static final int MAX_FRAME_HEIGHT = 360;

//  private static final int MAX_FRAME_WIDTH = 1200;
//  private static final int MAX_FRAME_HEIGHT = 675;

  private static CameraManager cameraManager;
  private static Camera.Parameters para;

  private final CameraConfigurationManager configManager;
  private static Camera camera;
  private Rect framingRect;
  private Rect framingRectInPreview;
  private boolean initialized;
  private boolean previewing;
  private final boolean useOneShotPreviewCallback;
  private final PreviewCallback previewCallback;
  private final AutoFocusCallback autoFocusCallback;
  
  static final int SDK_INT; // Later we can use Build.VERSION.SDK_INT
  static {
    int sdkInt;
    try {
      sdkInt = Integer.parseInt(Build.VERSION.SDK);
    } catch (NumberFormatException nfe) {
      // Just to be safe
      sdkInt = 10000;
    }
    SDK_INT = sdkInt;
  }

  
  public static void init(Context paramContext)
  {
    if (cameraManager == null) {
      cameraManager = new CameraManager(paramContext);
    }
  }
  
  public static CameraManager get()
  {
    return cameraManager;
  }
  
  private CameraManager(Context paramContext)
  {
    this.configManager = new CameraConfigurationManager(paramContext);
    
    this.useOneShotPreviewCallback = (Integer.parseInt(Build.VERSION.SDK) > 3);
    
    this.previewCallback = new PreviewCallback(this.configManager, this.useOneShotPreviewCallback);
    this.autoFocusCallback = new AutoFocusCallback();
  }



//this is new
  public void openDriver(SurfaceHolder paramSurfaceHolder)
    throws IOException
  {
    if (camera == null)
    {
      if (b4aZXingLib.useFrontCam)
      {
        int cameraId = findFrontFacingCamera();
        if (cameraId < 0) {
          BA.Log("no device");
        } else {
          camera = Camera.open(cameraId);
        }
      }
      else
      {
        camera = Camera.open();
      }
      if (camera == null) {
        throw new IOException();
      }
      camera.setPreviewDisplay(paramSurfaceHolder);
      if (!this.initialized)
      {
        this.initialized = true;
        this.configManager.initFromCameraParameters(camera);
      }
      this.configManager.setDesiredCameraParameters(camera);

      FlashlightManager.enableFlashlight();
    }
  }


  private int findFrontFacingCamera()
  {
    int cameraId = -1;
    
    int numberOfCameras = Camera.getNumberOfCameras();
    for (int i = 0; i < numberOfCameras; i++)
    {
      Camera.CameraInfo info = new Camera.CameraInfo();
      Camera.getCameraInfo(i, info);
      if (info.facing == 1)
      {
        cameraId = i;
        break;
      }
    }
    return cameraId;
  }





//this is old  
//  public void openDriver(SurfaceHolder paramSurfaceHolder)
//    throws IOException
//  {
//    if (camera == null)
//    {
//      camera = Camera.open();
//      if (camera == null) {
//        throw new IOException();
//      }
//      camera.setPreviewDisplay(paramSurfaceHolder);
//      if (!this.initialized)
//      {
//        this.initialized = true;
//        this.configManager.initFromCameraParameters(camera);
//      }
//      this.configManager.setDesiredCameraParameters(camera);
      
//      FlashlightManager.enableFlashlight();
//    }
//  }


  
  public void closeDriver()
  {
    if (camera != null)
    {
      myTorchParametersOff();
      camera.release();
      camera = null;
    }
  }


//this is old  
//  public void startPreview()
//  {
//    if ((camera != null) && (!this.previewing))
//    {
//      myTorchParametersOn();
//      camera.startPreview();
//      this.previewing = true;
//    }
//  }


//this is new
  public void startPreview()
  {
    if ((camera != null) && (!this.previewing))
    {
      if (b4aZXingLib.isportrait) {
        camera.setDisplayOrientation(90);
      }
      camera.startPreview();
      this.previewing = true;
    }
  }




  
  public static void myTorchParametersOn()
  {
    Camera.Parameters localParameters = camera.getParameters();
    localParameters.setFlashMode("torch");
    camera.setParameters(localParameters);
  }
  
  public static void myTorchParametersOff()
  {
    Camera.Parameters localParameters = camera.getParameters();
    localParameters.setFlashMode("off");
    camera.setParameters(localParameters);
  }
  
  public void stopPreview()
  {
    if ((camera != null) && (this.previewing))
    {
      if (!this.useOneShotPreviewCallback) {
        camera.setPreviewCallback(null);
      }
      myTorchParametersOff();
      camera.stopPreview();
      this.previewCallback.setHandler(null, 0);
      this.autoFocusCallback.setHandler(null, 0);
      this.previewing = false;
    }
  }
  
  public void requestPreviewFrame(Handler paramHandler, int paramInt)
  {
    if ((camera != null) && (this.previewing))
    {
      this.previewCallback.setHandler(paramHandler, paramInt);
      if (this.useOneShotPreviewCallback) {
        camera.setOneShotPreviewCallback(this.previewCallback);
      } else {
        camera.setPreviewCallback(this.previewCallback);
      }
    }
  }
  
  public void requestAutoFocus(Handler paramHandler, int paramInt)
  {
    if ((camera != null) && (this.previewing))
    {
      this.autoFocusCallback.setHandler(paramHandler, paramInt);
      
      camera.autoFocus(this.autoFocusCallback);
    }
  }
  
  public Rect getFramingRect()
  {
    Point localPoint = this.configManager.getScreenResolution();
    if (this.framingRect == null)
    {
      if (camera == null) {
        return null;
      }
      int i = localPoint.x * 3 / 4;
      if (i < 240) {
        i = 240;
      } else {
        i = (int) (localPoint.x * b4aZXingLib.theViewFinderXfactor);
        if (i > localPoint.x) {
          i = localPoint.x;
        } 
        if (i < 240) {
          i = 240;
        }  
      }
      int j = localPoint.y * 3 / 4;
      if (j < 240) {
        j = 240;
      } else {
        j = (int) (localPoint.y * b4aZXingLib.theViewFinderYfactor);
        if (j > localPoint.y) {
          j = localPoint.y;
        } 
        if (j < 240) {
          j = 240;
        }
      }
      int k = (localPoint.x - i) / 2;
      int m = (localPoint.y - j) / 2;
      this.framingRect = new Rect(k, m, k + i, m + j);
      BA.Log("Screen " + localPoint.x + " " + localPoint.y);


      Log.d(TAG, "Calculated framing rect: " + this.framingRect);
    }
    return this.framingRect;
  }
  
  public Rect getFramingRectInPreview()
  {
    if (this.framingRectInPreview == null)
    {
      Rect localRect = new Rect(getFramingRect());
      Point localPoint1 = this.configManager.getCameraResolution();
      Point localPoint2 = this.configManager.getScreenResolution();
      localRect.left = (localRect.left * localPoint1.y / localPoint2.x);
      localRect.right = (localRect.right * localPoint1.y / localPoint2.x);
      localRect.top = (localRect.top * localPoint1.x / localPoint2.y);
      localRect.bottom = (localRect.bottom * localPoint1.x / localPoint2.y);
      this.framingRectInPreview = localRect;
    }
    return this.framingRectInPreview;
  }
  
  public PlanarYUVLuminanceSource buildLuminanceSource(byte[] paramArrayOfByte, int paramInt1, int paramInt2)
  {
    Rect localRect = getFramingRectInPreview();
    int i = this.configManager.getPreviewFormat();
    String str = this.configManager.getPreviewFormatString();
    switch (i)
    {
    case 16: 
    case 17: 
      return new PlanarYUVLuminanceSource(paramArrayOfByte, paramInt1, paramInt2, localRect.left, localRect.top, localRect.width(), localRect.height());
    }
    if ("yuv420p".equals(str)) {
      return new PlanarYUVLuminanceSource(paramArrayOfByte, paramInt1, paramInt2, localRect.left, localRect.top, localRect.width(), localRect.height());
    }
    throw new IllegalArgumentException("Unsupported picture format: " + i + '/' + str);
  }
}
