
package ice.zxing.view;

import ice.zxing.camera.CameraManager;
import java.util.Collection;
import java.util.HashSet;
import java.lang.Math;
import android.content.Context;
import android.graphics.Bitmap;
import android.graphics.Canvas;
import android.graphics.Color;
import android.graphics.Paint;
import android.graphics.Rect;
import android.util.AttributeSet;
import android.view.View;
import android.widget.TextView;
import android.app.Activity;
import android.view.ViewParent;
import android.view.ViewGroup.LayoutParams;
import android.view.ViewGroup;
import android.graphics.Typeface;
import anywheresoftware.b4a.BA.Hide;
import anywheresoftware.b4a.BA;

import com.google.zxing.ResultPoint;
import ice.zxing.b4aZXingLib;

@Hide
public final class ViewfinderView extends View {

  private static final int[] SCANNER_ALPHA = {0, 64, 128, 192, 255, 192, 128, 64};
  private static final long ANIMATION_DELAY = 100L;
  private static final int OPAQUE = 0xFF;
  private static int countsPerSecond = 0;

  private final Paint paint;
  private Bitmap resultBitmap;
  private final int maskColor;
  private final int resultColor;
  private final int frameColor;
  private final int laserColor;
  private final int resultPointColor;
  private int scannerAlpha;
  private Collection<ResultPoint> possibleResultPoints;
  private Collection<ResultPoint> lastPossibleResultPoints;
  TextView textView;
  Context context;
  LayoutParams textViewLayoutParams;
  ViewGroup parentView;
  Rect dstRectForRender;

//        public static int theLaserColor;
//        public static int theMaskColor;
//        public static int theFrameColor;


  // This constructor is used when the class is built from an XML resource.
  public ViewfinderView(Context context, AttributeSet attrs) {
    super(context, attrs);

    // Initialize these once for performance rather than calling them every time in onDraw().
    paint = new Paint();

      //added by johan schoeman 
      if (b4aZXingLib.theMaskColor != 0) {
        maskColor = b4aZXingLib.theMaskColor;
      } else {
        maskColor = Color.argb(96, 0, 0, 0);
      }

      if (b4aZXingLib.theResultColor != 0) {
        resultColor = b4aZXingLib.theResultColor;
      } else {
        resultColor = Color.argb(176, 0, 0, 0);
      }

      if (b4aZXingLib.theFrameColor != 0) {
        frameColor = b4aZXingLib.theFrameColor;
      } else {
        frameColor = Color.argb(255, 0, 0, 0);
      }

      if (b4aZXingLib.theLaserColor != 0) {
        laserColor = b4aZXingLib.theLaserColor;
      } else {
        laserColor = Color.argb(255, 255, 0, 0);
      }

      if (b4aZXingLib.theResultPointColor != 0) {
        resultPointColor = b4aZXingLib.theResultPointColor;
      } else {
        resultPointColor = Color.argb(192,255,255,0);
      }
      //end of added by johan schoeman     


    scannerAlpha = 0;
    possibleResultPoints = new HashSet<ResultPoint>(5);
  }

  @Override
  public void onDraw(Canvas canvas) {
    Rect frame = CameraManager.get().getFramingRect();
    if (frame == null) {
      return;
    }

    int width = canvas.getWidth();
    int height = canvas.getHeight();


    // Draw the exterior (i.e. outside the framing rect) darkened
    paint.setColor(resultBitmap != null ? resultColor : maskColor);
    canvas.drawRect(0, 0, width, frame.top, paint);
    canvas.drawRect(0, frame.top, frame.left, frame.bottom + 1, paint);
    canvas.drawRect(frame.right + 1, frame.top, width, frame.bottom + 1, paint);
    canvas.drawRect(0, frame.bottom + 1, width, height, paint);


//added by johan schoeman
    paint.setColor(b4aZXingLib.bottomPromptColor);
    paint.setTextSize(b4aZXingLib.theBottomPromptTextSize); 
    paint.setTextSkewX(-0.25f);  //set Italic text
    paint.setTypeface(Typeface.defaultFromStyle(Typeface.BOLD));
//    paint.setTypeface(Typeface.MONOSPACE); working but stretching text too much
    int textlength = (int) (paint.measureText(b4aZXingLib.theBottomPromptMessage, 0, b4aZXingLib.theBottomPromptMessage.length()));
    int xPos = (int) ((width - textlength)/2);
    canvas.drawText(b4aZXingLib.theBottomPromptMessage, xPos, height - b4aZXingLib.bottomPromptDistanceFromBottom, paint);


    paint.setColor(b4aZXingLib.topPromptColor);
    paint.setTextSize(b4aZXingLib.theTopPromptTextSize); 
    paint.setTextSkewX(-0.25f);   //set Italic text
    textlength = (int) (paint.measureText(b4aZXingLib.theTopPromptMessage, 0, b4aZXingLib.theTopPromptMessage.length()));
    xPos = (int) ((width - textlength)/2);
    canvas.drawText(b4aZXingLib.theTopPromptMessage, xPos, b4aZXingLib.theTopPromptTextSize + b4aZXingLib.topPromptDistanceFromTop, paint);

//the following to be investigated to add different style text to the top and bottom messages

//   Typeface tf = Typeface.create("Helvetica",Typeface.BOLD);
//   Paint paint = new Paint();
//   paint.setTypeface(tf);
//   canvas.drawText("Sample text in bold Helvetica",0,0,paint);

//or

//Typeface plain = Typeface.createFromAsset(assetManager, pathToFont); 
//Typeface bold = Typeface.create(plain, Typeface.DEFAULT_BOLD;
//getContext().getAssets() ?????????

//or

// Paint paint = new Paint();
// paint.setTypeface(tf);
// paint.setTextSize(yourTextSize);
// canvas.drawText("Lorem ipsum", 0, 0, paint);

    if (b4aZXingLib.theBitMap != null) {
      Rect dstRectForRender = new Rect( b4aZXingLib.theBitMapLeft, 
                                        b4aZXingLib.theBitMapTop, 
                                        b4aZXingLib.theBitMapLeft + b4aZXingLib.theBitMapWidth,
                                        b4aZXingLib.theBitMapTop + b4aZXingLib.theBitMapHeight );
      canvas.drawBitmap (b4aZXingLib.theBitMap, null, dstRectForRender, null);
    }

//end of added by johan schoeman


    if (resultBitmap != null) {
      // Draw the opaque result bitmap over the scanning rectangle
      paint.setAlpha(OPAQUE);
      canvas.drawBitmap(resultBitmap, frame.left, frame.top, paint);
    } else {

      // Draw a two pixel solid black border inside the framing rect
      paint.setColor(frameColor);
      canvas.drawRect(frame.left, frame.top, frame.right + 1, frame.top + 2, paint);
      canvas.drawRect(frame.left, frame.top + 2, frame.left + 2, frame.bottom - 1, paint);
      canvas.drawRect(frame.right - 1, frame.top, frame.right + 1, frame.bottom - 1, paint);
      canvas.drawRect(frame.left, frame.bottom - 1, frame.right + 1, frame.bottom + 1, paint);

      // Draw a red "laser scanner" line through the middle to show decoding is active
//      countsPerSecond += 1;
//      BA.Log("drawing laser" + countsPerSecond); timed this to be about 8 times per second
      paint.setColor(laserColor);
      paint.setAlpha(SCANNER_ALPHA[scannerAlpha]);
      scannerAlpha = (scannerAlpha + 1) % SCANNER_ALPHA.length;
      int middle = frame.height() / 2 + frame.top;
      canvas.drawRect(frame.left + 2, middle - 1, frame.right - 1, middle + 2, paint);

      Collection<ResultPoint> currentPossible = possibleResultPoints;
      Collection<ResultPoint> currentLast = lastPossibleResultPoints;
      if (currentPossible.isEmpty()) {
        lastPossibleResultPoints = null;
      } else {
        possibleResultPoints = new HashSet<ResultPoint>(5);
        lastPossibleResultPoints = currentPossible;
        paint.setAlpha(OPAQUE);
        paint.setColor(resultPointColor);
        for (ResultPoint point : currentPossible) {
          canvas.drawCircle(frame.left + point.getX(), frame.top + point.getY(), 6.0f, paint);
        }
      }
      if (currentLast != null) {
        paint.setAlpha(OPAQUE / 2);
        paint.setColor(resultPointColor);
        for (ResultPoint point : currentLast) {
          canvas.drawCircle(frame.left + point.getX(), frame.top + point.getY(), 3.0f, paint);
        }
      }

      // Request another update at the animation interval, but only repaint the laser line,
      // not the entire viewfinder mask.
      postInvalidateDelayed(ANIMATION_DELAY, frame.left, frame.top, frame.right, frame.bottom);
    }
  }

  public void drawViewfinder() {
    resultBitmap = null;
    invalidate();
  }

  /**
   * Draw a bitmap with the result points highlighted instead of the live scanning display.
   *
   * @param barcode An image of the decoded barcode.
   */
  public void drawResultBitmap(Bitmap barcode) {
    resultBitmap = barcode;
    invalidate();
  }

  public void addPossibleResultPoint(ResultPoint point) {
    possibleResultPoints.add(point);
  }

}
